(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
typeof define === 'function' && define.amd ? define(['exports'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.shapey = {}));
}(this, (function (exports) { 'use strict';

function getDefaultExportFromCjs (x) {
	return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, basedir, module) {
	return module = {
		path: basedir,
		exports: {},
		require: function (path, base) {
			return commonjsRequire(path, (base === undefined || base === null) ? module.path : base);
		}
	}, fn(module, module.exports), module.exports;
}

function commonjsRequire () {
	throw new Error('Dynamic requires are not currently supported by @rollup/plugin-commonjs');
}

var curry_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Takes a Function whose params are meant to be supplied all at once and changes it so they can be supplied one at a time.
 * As each argument is supplied a new Function is returned that is ready to receive the next argument.
 * This continues until all arguments for your origianl function have been supplied and then the actual result is returned.
 *
 * Note: you cannot set default values for curried function params (again, you _cannot_ set default values for curried function params)
 *
 * @function
 * @param {Function} fn A Function whose signature needs to changed from requiring all at once to providing them one (or more) at a time.
 * @returns {Function} A new Function that will wait until all arguments have been supplied before returning a result (otherwise it will continue to return a new Function that is ready to receive the next argument)
 */
function curry(fn) {
  var fnlen = fn.length;

  switch (fnlen) {
    case 2:
      return function standardCurry(a, b) {
        if (arguments.length === 2) {
          return fn(a, b);
        }

        return function (_b) {
          return fn(a, _b);
        };
      };

    case 3:
      return function threeArgCurry(a, b, c) {
        switch (arguments.length) {
          case 3:
            return fn(a, b, c);

          case 2:
            return function (_c) {
              return fn(a, b, _c);
            };

          default:
            return curry(function (_b, _c) {
              return fn(a, _b, _c);
            });
        }
      };

    case 4:
      return function fourArgCurry(a, b, c, d) {
        switch (arguments.length) {
          case 4:
            return fn(a, b, c, d);

          case 3:
            return function (_d) {
              return fn(a, b, c, _d);
            };

          case 2:
            return curry(function (_c, _d) {
              return fn(a, b, _c, _d);
            });

          default:
            return curry(function (_b, _c, _d) {
              return fn(a, _b, _c, _d);
            });
        }
      };

    default:
      return function manyArgCurry() {
        for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        var arglen = args.length;

        if (arglen === fnlen) {
          return fn.apply(void 0, args);
        }

        return function inner() {
          for (var _len2 = arguments.length, innerArgs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            innerArgs[_key2] = arguments[_key2];
          }

          var innerlen = innerArgs.length;
          var limit = fnlen;
          var allArgs = [];
          var argCount = 0;

          while (argCount < arglen) {
            allArgs.push(args[argCount]);
            argCount++;
            limit--;
          }

          var innerArgCount = 0;

          while (innerArgCount < innerlen) {
            allArgs.push(innerArgs[innerArgCount]);
            innerArgCount++;
            limit--;
          }

          return limit > 0 ? manyArgCurry.apply(void 0, allArgs) : fn.apply(void 0, allArgs);
        };
      };
  }
}

var _default = curry;
exports.default = _default;
module.exports = exports.default;
});

var curry = /*@__PURE__*/getDefaultExportFromCjs(curry_1);

var has_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Checks if a given Object contains a specified prop name
 *
 * @function
 * @param {String} key A prop name to look for in the object
 * @param {object} obj An Object to inspect for a given prop
 * @returns {Boolean} Whether the object contains the specified prop
 */
function has(key, obj) {
  return Object.prototype.hasOwnProperty.call(obj, key);
}

var _default = has;
exports.default = _default;
module.exports = exports.default;
});

var has = /*@__PURE__*/getDefaultExportFromCjs(has_1);

var forIn_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _has = _interopRequireDefault(has_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A light wrapper around native `for .. in`, but will only iterate over an Object's own properties.
 *
 * @function
 * @param {Function} fn A function to execute iteratively, which will receive the `key`, `value`, and `object` (respectively)
 * @param {object} obj An object whose keys will be iterated over
 */
function forIn(fn, obj) {
  // eslint-disable-next-line no-restricted-syntax
  for (var key in obj) {
    if ((0, _has.default)(key, obj)) {
      fn(key, obj[key], obj);
    }
  }
}

var _default = forIn;
exports.default = _default;
module.exports = exports.default;
});

var merge_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Merges the values from 2 or more Objects or Arrays together into a new Object/Array.
 * Null and Undefined values are handled gracefully, and if the second value is a primitive it will be returned as-is, instead of trying to merge it onto the first.
 *
 * @function
 * @param {object|Array<*>} val The first value to merge onto (will not get mutated though)
 * @param {object|Array<*>} val2 A value to merge onto the first
 * @returns {object|Array<*>} A new value that contains the combined values from all the values passed in
 */
function merge() {
  for (var _len = arguments.length, vals = new Array(_len), _key = 0; _key < _len; _key++) {
    vals[_key] = arguments[_key];
  }

  var numOfVals = vals.length;

  if (vals[0] === undefined || vals[1] === undefined) {
    return vals[1] !== undefined ? vals[1] : vals[0];
  }

  if (numOfVals === 2 && (typeof vals[1] !== "object" || !vals[1] || vals[1].constructor.name !== "Object")) {
    return vals[1];
  }

  var newObj = {};
  (0, _forIn.default)(function (key, val) {
    newObj[key] = val;
  }, vals[0]);

  for (var i = 1; i < numOfVals; i++) {
    (0, _forIn.default)(function (key, val) {
      newObj[key] = merge(newObj[key], val);
    }, vals[i]);
  }

  return newObj;
}

var _default = merge;
exports.default = _default;
module.exports = exports.default;
});

var merge = /*@__PURE__*/getDefaultExportFromCjs(merge_1);

var append_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _merge = _interopRequireDefault(merge_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Blends two values together based on their type.
 * If both are objects this would result in a new object with the second value's props merged onto the first's.
 * If the first value is an array, this would result in a new array with the second value concatenated onto the first value.
 * If both values are strings or numbers, a new string will be returned with the second value added onto the first.
 *
 * @function
 * @param {String|Number|Object<string, any>|Array<*>} firstVal A value that will have another appended onto
 * @param {String|Number|Object<string, any>|Array<*>} secondVal A value to append to the first value
 * @returns {Array<*>|object|String} A new Array, Object, or String that has the characters/values from the second provided value merged _after_ those from the first provided value
 */
function append(firstVal, secondVal) {
  if ([firstVal, secondVal].every(function (val) {
    return typeof val === "string" || typeof val === "number";
  })) {
    return "" + firstVal + secondVal;
  }

  if (Array.isArray(firstVal) && Array.isArray(secondVal)) {
    return [].concat(firstVal, secondVal);
  }

  return (0, _merge.default)(firstVal, secondVal);
}

var _default = append;
exports.default = _default;
module.exports = exports.default;
});

var cond_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Several pieces of conditional logic to apply against a value and the _first_ one which matches will have a corresponding transformation applied to it.
 *
 * Supply an Array of pairs:
 *   - the first value is the conditional logic (`Function`) to match against the value
 *   - the second value is the transformation (`Function`) to apply to the value if the condition was matched
 *
 * __Note__: if your tranformation is _not_ a function then it will returned as-is in response to a succesfully met condition
 *
 * @function
 * @param {Array<Array<Function>>} conditionalTransforms An array of arrays (which have two values: the condition function and the transformation function)
 * @param {*} val A value of any type that will be transformed according to the appropriate condition.
 * @returns {*} The provided value transformed by the appropriate matching conditional transformation
 */
function cond(conditionalTransforms, val) {
  var result;
  conditionalTransforms.some(function (_ref) {
    var ifCondition = _ref[0],
        _ref$ = _ref[1],
        transform = _ref$ === void 0 ? val : _ref$;

    if (ifCondition(val)) {
      result = typeof transform === "function" ? transform(val) : transform;
      return true;
    }

    return false;
  });
  return result;
}

var _default = cond;
exports.default = _default;
module.exports = exports.default;
});

var _includes_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports._includes = _includes;

function _includes(val, arr) {
  var idx = arr.length;

  while (idx--) {
    if (arr[idx] === val) {
      return true;
    }
  }

  return false;
}
});

var contains_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;



/**
 * Checks if a given value is present in a String OR Array
 *
 * @function
 * @param {String|Number} val A value which may be present in the String/Array
 * @param {Array<*>|String} arr An Array or String which may contain the provided value
 * @returns {Boolean} Whether or not the String|Array contains the provided value
 */
function contains(val, arr) {
  return Array.isArray(arr) ? (0, _includes_1._includes)(val, arr) : arr.indexOf(val) !== -1;
}

var _default = contains;
exports.default = _default;
module.exports = exports.default;
});

var difference_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;



/**
 * Compares two lists of Strings/Numbers and returns the values that are different between the two lists
 *
 * @function
 * @param {Array<String>|Array<Number>} arr1 An Array of Strings
 * @param {Array<String>|Array<Number>} arr2 An Array of Strings
 * @returns {Array<String>|Array<Number>} An array of values that are different between the two lists
 */
function difference(arr1, arr2) {
  var diff = [];
  var len1 = arr1.length;
  var len2 = arr2.length;

  if (len1 === 0 && len2 === 0) {
    return diff;
  } else if (len1 === 0) {
    return arr2;
  } else if (len2 === 0) {
    return arr1;
  }

  for (var i = 0; i < len1; i++) {
    var val = arr1[i];

    if (!(0, _includes_1._includes)(val, arr2)) {
      diff.push(val);
    }
  }

  for (var j = 0; j < len2; j++) {
    var _val = arr2[j];

    if (!(0, _includes_1._includes)(_val, arr1)) {
      diff.push(_val);
    }
  }

  return diff;
}

var _default = difference;
exports.default = _default;
module.exports = exports.default;
});

var difference = /*@__PURE__*/getDefaultExportFromCjs(difference_1);

var each_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * A faster forEach that provides the same API as native.
 *
 * @function
 * @param {Function} fn A Function to execute for each iteration. It will receive the value, index and full array (respectively) as args
 * @param {Array<*>} arr An Array to iterate over (any value will be passed into the iterate Function)
 */
function each(fn, arr) {
  for (var i = 0, count = arr.length; i < count; i++) {
    fn(arr[i], i, arr);
  }
}

var _default = each;
exports.default = _default;
module.exports = exports.default;
});

var find_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Find a single value from an array of values, based on criteria defined in a predicate function.
 *
 * @function
 * @param {Function} pred A predicate function to apply to the array of values (It should take a val as input and return a Boolean as output).
 * @param {Array<*>} arr An array of values from which to find one particular matching value
 * @returns {*} Either a value from the array that matched the predicate function or undefined (if no match)
 */
function find(pred, arr) {
  var len = arr.length;

  for (var i = 0; i < len; i++) {
    if (pred(arr[i])) return arr[i];
  }
}

var _default = find;
exports.default = _default;
module.exports = exports.default;
});

var findIndex_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Find the index of a single value from an array of values, based on criteria defined in a predicate function.
 *
 * @function
 * @param {Function} pred A predicate function to apply to the array of values (It should take a val as input and return a Boolean as output).
 * @param {Array<*>} arr An array of values from which to find the index of one particular matching value
 * @returns {Number} Either the index of the value from the array that matched the predicate function or negative one (-1, if no match).
 */
function findIndex(pred, arr) {
  var i = -1;
  var len = arr.length;

  while (++i < len) {
    if (pred(arr[i])) return i;
  }

  return -1;
}

var _default = findIndex;
exports.default = _default;
module.exports = exports.default;
});

var isNil_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Checks to see if a value is null OR undefined
 *
 * @function
 * @param {*} val A value (of any type)
 * @returns {Boolean} Whether or not the value is null or undefined
 */
function isNil(val) {
  return val == null;
}

var _default = isNil;
exports.default = _default;
module.exports = exports.default;
});

var getConstructorName_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

function getConstructorName(val) {
  try {
    return val.constructor.name;
  } catch (err) {
    return "";
  }
}

var _default = getConstructorName;
exports.default = _default;
module.exports = exports.default;
});

var isArrayish_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isNil = _interopRequireDefault(isNil_1);

var _getConstructorName = _interopRequireDefault(getConstructorName_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isArrayLike(name) {
  switch (name) {
    case "Set":
    case "WeakSet":
    case "Array":
    case "Float64Array":
    case "Float32Array":
    case "Int32Array":
    case "Uint16Array":
    case "Int16Array":
    case "Uint8ClampedArray":
    case "Uint8Array":
    case "Int8Array":
      return true;

    default:
      return false;
  }
}
/**
 * Checks if a given value is "array-like".
 *
 * This includes:
 * - Array
 * - Set
 * - WeakSet
 * - Float64Array
 * - Float32Array
 * - Int32Array
 * - Uint16Array
 * - Int16Array
 * - Uint8ClampedArray
 * - Uint8Array
 * - Int8Array
 *
 * @function
 * @param {*} val A value to check as being an array
 * @returns {Boolean} Whether the value is an array-like type
 */


function isArrayish(val) {
  return Array.isArray(val) || !(0, _isNil.default)(val) && isArrayLike((0, _getConstructorName.default)(val));
}

var _default = isArrayish;
exports.default = _default;
module.exports = exports.default;
});

var isUndefined_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Checks to see if a value is undefined
 *
 * @function
 * @param {*} val A value (of any type)
 * @returns {Boolean} Whether or not the value is undefined
 */
function isUndefined(val) {
  return val === undefined;
}

var _default = isUndefined;
exports.default = _default;
module.exports = exports.default;
});

var isObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isNil = _interopRequireDefault(isNil_1);

var _isUndefined = _interopRequireDefault(isUndefined_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function isAnythingButAnObject(name) {
  switch (name) {
    case "Function":
    case "RegExp":
    case "Date":
    case "WeakMap":
    case "Map":
    case "WeakSet":
    case "Set":
    case "Float64Array":
    case "Float32Array":
    case "Int32Array":
    case "Uint16Array":
    case "Int16Array":
    case "Uint8ClampedArray":
    case "Uint8Array":
    case "Int8Array":
    case "Array":
    case "Symbol":
    case "Number":
    case "String":
    case "Boolean":
    case "UriError":
    case "TypeError":
    case "SyntaxError":
    case "ReferenceError":
    case "RangeError":
    case "InternalError":
    case "EvalError":
    case "Error":
      return true;

    default:
      return false;
  }
}
/**
 * A high-speed, mostly adequate check of a value which may be an Object.
 * This excludes values that are _technically_ an Object but in practice are not what you _really_ mean when you speak of Objects.
 *
 * @function
 * @param {*} val A value (of any type)
 * @returns {Boolean} Whether or not the value is an Object
 */


function isObject(val) {
  return !(0, _isNil.default)(val) && (0, _isUndefined.default)(val.length) && (!(0, _isUndefined.default)(val.constructor) && !isAnythingButAnObject(val.constructor.name) || (0, _isUndefined.default)(val.constructor) && (0, _isUndefined.default)(val.prototype));
}

var _default = isObject;
exports.default = _default;
module.exports = exports.default;
});

var isObject = /*@__PURE__*/getDefaultExportFromCjs(isObject_1);

var filterString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Applies a filtering function you provide over every character in a given string.
 *
 * @function
 * @param {Function} fn A filtering function that is invoked on every char in the provided String value
 * @param {String} str A string value to filter over
 * @returns {String} A new String value that is the result of the filtering operation over the original string
 */
function filterString(fn, str) {
  var len = str.length;
  var newStr = "";

  for (var i = 0; i < len; i++) {
    newStr += fn(str[i], i, str) ? str[i] : "";
  }

  return newStr;
}

var _default = filterString;
exports.default = _default;
module.exports = exports.default;
});

var filterObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies a filtering function you provide over every value in a given Object.
 *
 * @function
 * @param {Function} fn A filtering function that is invoked on every value in the provided Object
 * @param {object} obj An Object whose values will be filtered
 * @returns {object} A new Object that is the result of the filtering operation over all the values in the original Object
 */
function filterObject(fn, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val, ob) {
    if (fn(val, key, ob)) {
      newObj[key] = val;
    }
  }, obj);
  return newObj;
}

var _default = filterObject;
exports.default = _default;
module.exports = exports.default;
});

var filter_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isArrayish = _interopRequireDefault(isArrayish_1);

var _isObject = _interopRequireDefault(isObject_1);

var _filterString = _interopRequireDefault(filterString_1);

var _filterObject = _interopRequireDefault(filterObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies a filtering function you provide over a value you provide, according to its type.
 * String values will have the filtering function applied over every character in the String.
 * Objects will have the filtering function applied to every value in the Object.
 * Arrays (or Array-like values) will have the filtering function applied to every value in the Array.
 * If the type of your value is none of the above, the value will be returned as-is.
 *
 * @function
 * @param {Function} fn A filtering function that is invoked on the provided value
 * @param {object|Array<*>|String} val An Object/Array/String whose values/chars will be filtered
 * @returns {object|Array<*>|String} A new value that is the result of the filtering operation over all the chars or values in the original String/Object/Array
 */
function filter(fn, val) {
  if (Array.isArray(val)) return val.filter(fn);
  if ((0, _isObject.default)(val)) return (0, _filterObject.default)(fn, val);
  if (typeof val === "string") return (0, _filterString.default)(fn, val);
  if ((0, _isArrayish.default)(val)) return val.filter(fn);
  return val;
}

var _default = filter;
exports.default = _default;
module.exports = exports.default;
});

var filter = /*@__PURE__*/getDefaultExportFromCjs(filter_1);

var fuzzy_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Performs a fuzzy search on a list of strings or objects.
 * If a list of objects, provided the prop extraction function so the search can find the correct field(s)
 * This is heavily inspired by (most of) the algorithm used by [Matt York's](https://github.com/myork/fuzzy) fuzzy search function,
 * however several features were not carried over and his implementation of that alrgorithm has been significantly changed to achieve a 25% speed improvement.
 * Please see his original work - called [fuzzy](https://www.npmjs.com/package/fuzzy) MIT - if you need some of his additional options.
 *
 * @function
 * @param {Function} [propFn] A function which will extract all the fields which you wish to fuzzy search on. Omit if the list is a list of strings
 * @param {string} needle The search value itself
 * @param {boolean} [caseSensitive=false] Whether or not to perform a case-sensitive search
 * @param {Array<String>|Array<object>} arr An array of string values or objects which have string values to be searched on
 * @returns {Array<String>|Array<object>} The filtered list of search results
 */
function fuzzy(propFn, needle, caseSensitive, arr) {
  if (arr == null || !Array.isArray(arr) || arr.length === 0) return [];
  if (typeof needle !== "string" || !needle.trim().length) return arr;
  var scores = [];
  var len = arr.length;
  var needleLen = needle.length;
  var idx = len;
  var noodle = caseSensitive ? needle : needle.toLowerCase();
  var extractFn = typeof propFn !== "function" && caseSensitive ? function (v) {
    return v;
  } : typeof propFn !== "function" ? function (v) {
    return v.toLowerCase();
  } : caseSensitive ? propFn : function (v) {
    return propFn(v).toLowerCase();
  };

  while (--idx) {
    var score = 0;
    var val = extractFn(arr[idx]);
    var valLen = val.length;

    if (noodle === val) {
      scores.push([Infinity, idx]);
    } else if (valLen !== 0) {
      var valIdx = 0;
      var matchedInNeedleIdx = 0;
      var numOfCharsMatchedAtOnce = 0;

      while (valIdx < valLen) {
        if (noodle[matchedInNeedleIdx] === val[valIdx]) {
          matchedInNeedleIdx++;
          numOfCharsMatchedAtOnce++;
        } else {
          numOfCharsMatchedAtOnce = 0;
        }

        score += numOfCharsMatchedAtOnce;
        valIdx++;
      }

      if (matchedInNeedleIdx === needleLen) {
        scores.push([score, idx]);
      }
    }
  }

  return scores.sort(function (a, b) {
    return b[0] - a[0] || a[1] - b[1];
  }).map(function (_ref) {
    _ref[0];
        var index = _ref[1];
    return arr[index];
  });
}

var _default = fuzzy;
exports.default = _default;
module.exports = exports.default;
});

var hasNestedProp_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _has = _interopRequireDefault(has_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks if a given Object contains a (potentially) nested property of a specified path
 *
 * @function
 * @param {Array<String>|String} prop A prop name, a dot-separated prop path, or an array of prop path "pieces" to look for in the object
 * @param {object} obj An Object to inspect for a given prop at the specified path
 * @returns {Boolean} Whether the object contains the specified prop path
 */
function hasNestedProp(prop, obj) {
  var paths = typeof prop === "string" ? prop.split(".") : prop;
  var len = paths.length;
  if (len === 0) return false;
  var idx = 0;
  var val = obj[paths[idx]];
  var hasProp = (0, _has.default)(paths[idx], obj);

  while (++idx < len && hasProp === true) {
    hasProp = (0, _has.default)(paths[idx], val);

    if (hasProp) {
      val = val[paths[idx]];
    }
  }

  return hasProp;
}

var _default = hasNestedProp;
exports.default = _default;
module.exports = exports.default;
});

var intersection_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;



/**
 * Compares two lists of Strings/Numbers and returns the values that are in common (intersect) between the two lists
 *
 * @function
 * @param {Array<String>|Array<Number>} arr1 An Array of Strings/Numbers
 * @param {Array<String>|Array<Number>} arr2 An Array of Strings/Numbers
 * @returns {Array<String>|Array<Number>} The values in common between the two lists
 */
function intersection(arr1, arr2) {
  var diff = [];
  var len1 = arr1.length;
  var len2 = arr2.length;
  if (len1 === 0 || len2 === 0) return diff;

  for (var i = 0; i < len1; i++) {
    var val = arr1[i];

    if ((0, _includes_1._includes)(val, arr2)) {
      diff.push(val);
    }
  }

  for (var j = 0; j < len2; j++) {
    var _val = arr2[j];

    if ((0, _includes_1._includes)(_val, arr1) && !(0, _includes_1._includes)(_val, diff)) {
      diff.push(_val);
    }
  }

  return diff;
}

var _default = intersection;
exports.default = _default;
module.exports = exports.default;
});

var isStrictEqual_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Inspects two values to see if they are strictly equal, meaning no type coercion or deepyly nested equality checks are performed.
 * A very simple triple equals is all that is used.
 *
 * @function
 * @param {*} val1 A value (of any type)
 * @param {*} val2 A value (of any type)
 * @returns {Boolean} Whether or not the two values are strictly equal
 */
function isStrictEqual(firstVal, secondVal) {
  return firstVal === secondVal;
}

var _default = isStrictEqual;
exports.default = _default;
module.exports = exports.default;
});

var toTitleCase_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Transforms a string value into one which is title-cased. The first letter of any word is capitalized.
 *
 * @function
 * @param {String} str A string which may contain uppercase characters
 * @returns {String} A new string that is an lowercase representation of the original string
 */
function toTitleCase(str) {
  return str.split(/\s/).map(function (s) {
    return "" + s.charAt(0).toUpperCase() + s.slice(1);
  }).join(" ");
}

var _default = toTitleCase;
exports.default = _default;
module.exports = exports.default;
});

var getType_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isNil = _interopRequireDefault(isNil_1);

var _toTitleCase = _interopRequireDefault(toTitleCase_1);

var _getConstructorName = _interopRequireDefault(getConstructorName_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Gets the type for any value. If available will inspect the constructor name, otherwise will use the typeof
 *
 * @function
 * @param {*} val A value of any kind
 * @returns {String} The stringified representation of the value's type
 */
function getType(val) {
  if ((0, _isNil.default)(val)) {
    return "" + val;
  }

  return (0, _getConstructorName.default)(val) || (0, _toTitleCase.default)(typeof val);
}

var _default = getType;
exports.default = _default;
module.exports = exports.default;
});

var isSameType_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _getType = _interopRequireDefault(getType_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Inspects two values to see if they are the same type.
 * The typeof and (if necessary) constructor names are inspected during this check.
 *
 * @function
 * @param {*} val1 A value (of any type)
 * @param {*} val2 A value (of any type)
 * @returns {Boolean} Whether or not the two values are of the same type
 */
function isSameType(val1, val2) {
  return (0, _getType.default)(val1) === (0, _getType.default)(val2);
}

var _default = isSameType;
exports.default = _default;
module.exports = exports.default;
});

var isEqual_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isObject = _interopRequireDefault(isObject_1);

var _isArrayish = _interopRequireDefault(isArrayish_1);

var _isStrictEqual = _interopRequireDefault(isStrictEqual_1);

var _isSameType = _interopRequireDefault(isSameType_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks if two provided values are deeply equal.
 * If Objects or Arrays (or Array-like values) are provided, they are inspected recursively.
 * Primitive values are checked to see if they are stricly equal (ie triple equals; no type coercion).
 *
 * @function
 * @param {*} firstVal A value which may be null, undefined, a JavaScript primitive value, an array of values, an array-like value, or an object
 * @param {*} secondVal A value which may be null, undefined, a JavaScript
 * @returns {Boolean} Whether or not the two values are deeply equal
 */
function isEqual(firstVal, secondVal) {
  if ((0, _isStrictEqual.default)(firstVal, secondVal)) return true;
  if (!(0, _isSameType.default)(firstVal, secondVal)) return false;

  if (Array.isArray(firstVal) && Array.isArray(secondVal)) {
    var valLen = firstVal.length;
    if (valLen !== secondVal.length) return false;

    for (var i = 0; i < valLen; i++) {
      if (!isEqual(firstVal[i], secondVal[i])) return false;
    }

    return true;
  }

  if ((0, _isObject.default)(firstVal)) {
    var firstKeys = Object.keys(firstVal);
    var secondKeys = Object.keys(secondVal);
    return firstKeys.length === secondKeys.length && firstKeys.every(function (key) {
      return isEqual(firstVal[key], secondVal[key]);
    });
  }

  if ((0, _isArrayish.default)(firstVal)) {
    return firstVal.every(function (v, i) {
      return isEqual(v, secondVal[i]);
    });
  }

  return false;
}

var _default = isEqual;
exports.default = _default;
module.exports = exports.default;
});

var mapString_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Applies a mapping function you provide over every character in a given string.
 *
 * @function
 * @param {Function} fn A mapping function that is invoked on every char in the provided String value
 * @param {String} str A string value to map over
 * @returns {String} A new String value that is the result of the mapping operation over the original string
 */
function mapString(fn, str) {
  var len = str.length;
  var newStr = "";

  for (var i = 0; i < len; i++) {
    newStr += fn(str[i], i, str);
  }

  return newStr;
}

var _default = mapString;
exports.default = _default;
module.exports = exports.default;
});

var mapObject_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies a mapping function you provide over every value in a given Object.
 *
 * @function
 * @param {Function} fn A mapping function that is invoked on every value in the provided Object
 * @param {object} obj An Object whose values will be mapped over
 * @returns {object} A new Object that is the result of the mapping operation over all the values in the original Object
 */
function mapObject(fn, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val, ob) {
    newObj[key] = fn(val, key, ob);
  }, obj);
  return newObj;
}

var _default = mapObject;
exports.default = _default;
module.exports = exports.default;
});

var mapObject = /*@__PURE__*/getDefaultExportFromCjs(mapObject_1);

var map_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isArrayish = _interopRequireDefault(isArrayish_1);

var _isObject = _interopRequireDefault(isObject_1);

var _mapString = _interopRequireDefault(mapString_1);

var _mapObject = _interopRequireDefault(mapObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies a mapping function you provide over a value you provide, according to its type.
 * String values will have the mapping function applied over every character in the String.
 * Objects will have the mapping function applied to every value in the Object.
 * Arrays (or Array-like values) will have the mapping function applied to every value in the Array.
 * If the type of your value is none of the above, the value will be returned as-is.
 * Also, this mapping operation does _not_ mutate the original value.
 *
 * @function
 * @param {Function} fn A mapping function that is invoked on the provided value
 * @param {object|Array<*>|String} val An Object/Array/String whose values/chars will be mapped over
 * @returns {object|Array<*>|String} A new value that is the result of the mapping operation over all the chars or values in the original String/Object/Array
 */
function map(fn, val) {
  if (Array.isArray(val)) return val.map(fn);
  if ((0, _isObject.default)(val)) return (0, _mapObject.default)(fn, val);
  if (typeof val === "string") return (0, _mapString.default)(fn, val);
  if ((0, _isArrayish.default)(val)) return val.map(fn);
  return val;
}

var _default = map;
exports.default = _default;
module.exports = exports.default;
});

var map = /*@__PURE__*/getDefaultExportFromCjs(map_1);

var mapObjectRecursive_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);

var _isObject = _interopRequireDefault(isObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies a mapping function you provide over every value in a given Object (recursively).
 *
 * @function
 * @param {Function} fn A mapping function that is invoked on every value in the provided Object
 * @param {object} obj An Object whose values will be mapped over (recursively)
 * @returns {object} A new Object that is the result of the mapping operation over all the values in the original Object
 */
function mapObjectRecursive(fn, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val, ob) {
    /* Looks redundant, but is written this way for speed */
    if (typeof val === "object" && val != null && (val.constructor && val.constructor.name === "Object" || (0, _isObject.default)(val))) {
      newObj[key] = mapObjectRecursive(fn, val);
    } else {
      newObj[key] = fn(val, key, ob);
    }
  }, obj);
  return newObj;
}

var _default = mapObjectRecursive;
exports.default = _default;
module.exports = exports.default;
});

var mapSpec_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _has = _interopRequireDefault(has_1);

var _forIn = _interopRequireDefault(forIn_1);

var _isObject = _interopRequireDefault(isObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Applies one or more mapping functions to the value in a given Object.
 * But rather than applying the same mapping function to every value in an Object, instead you use a "spec" object to link the appropriate mapping function to the key/val in the input Object.
 *
 * This is similar to Ramda's `evolve()` however you can also set values in your spec that are _not_ functions (which will just override whatever matching key there might be on the input object).
 * Addtionally, it supplies the key and the object as the 2nd and third params to your spec's transformation function, so that you can create props based on the entire input Object (with Ramda you'll need to also use `applySpec()` and in a separate operation to derived these kinds of values).
 *
 * As an example, If you want a mapping function to be applied to a prop called "name", then you would first pass in a spec object with a prop on it called "name" and the value would be the mapping function.
 * Then the actual input object (with the desciption field) will have the matching mapping function from your spec applied to it directly.
 *
 * @function
 * @param {object} spec An Object whose keys should correspond to keys in the input Object and whose values are mapping functions that will receive the matching input Object's prop as input
 * @param {object} inputObj The actual input to map over and transform
 * @returns {object} A new Object with all the mapping functions from the spec Object applied to the corresponding values in the input Object (if they exist)
 * @example
 *   mapSpec({
 *     age: Number,
 *     isAlive: Boolean,
 *     name: str => `${str.charAt(0).toUpperCase()}${str.slice(1)}`
 *   }, {
 *     age: '20',
 *     isAlive: 1,
 *     name: 'john'
 *   })
 */
function mapSpec(spec, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val, ob) {
    if (typeof spec[key] === "function") {
      newObj[key] = spec[key](val, key, ob);
    } else if ((0, _isObject.default)(spec[key])) {
      newObj[key] = mapSpec(spec[key], val);
    } else {
      newObj[key] = val;
    }
  }, obj);
  (0, _forIn.default)(function (key, val) {
    if (typeof val !== "function") {
      newObj[key] = val;
    } else if (!(0, _has.default)(key, newObj)) {
      newObj[key] = val(undefined, key, newObj);
    }
  }, spec);
  return newObj;
}

var _default = mapSpec;
exports.default = _default;
module.exports = exports.default;
});

var omit_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);



function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Removes specified keys from an object (after cloning the Object).
 *
 * @function
 * @param {Array<string>} keys An array of keys to search for in the Object and exclude from the output
 * @param {object} obj An Object from which to copy and remove keys
 * @returns {object} A copy of the original Object, but without the specified keys
 */
function omit(keys, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val) {
    if (!(0, _includes_1._includes)(key, keys)) {
      newObj[key] = val;
    }
  }, obj);
  return newObj;
}

var _default = omit;
exports.default = _default;
module.exports = exports.default;
});

var omit = /*@__PURE__*/getDefaultExportFromCjs(omit_1);

var pick_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Removes everything _except_ the specified keys from an object (after cloning the Object).
 *
 * @function
 * @param {Array<String>} keys An array of keys to search for in the Object and include from the output
 * @param {object} obj An Object from which to copy and remove keys
 * @returns {object} A copy of the original Object, but with _only_ the specified keys
 */
function pick(keys, obj) {
  var newObj = {};
  var numOfKeys = keys.length;

  for (var i = 0; i < numOfKeys; i++) {
    if (keys[i] in obj) {
      newObj[keys[i]] = obj[keys[i]];
    }
  }

  return newObj;
}

var _default = pick;
exports.default = _default;
module.exports = exports.default;
});

var pick = /*@__PURE__*/getDefaultExportFromCjs(pick_1);

var prepend_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _merge = _interopRequireDefault(merge_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Merges two values together, placing the characters (or values) from one before those from the other.
 *
 * @function
 * @param {Array<*>|object|String|Number} firstVal An Array, Object, String or Number that the will have a new value(s) merged before its own characters/values
 * @param {Array<*>|object|String|Number} secondVal An Array, Object, String or Number that the will merge _before_ those from the first provided value
 * @returns {Array<*>|object|String} A new Array, Object, or String that has the characters/values from the second provided value merged _before_ those from the first provided value
 */
function prepend(firstVal, secondVal) {
  if ([firstVal, secondVal].every(function (val) {
    return typeof val === "string" || typeof val === "number";
  })) {
    return "" + secondVal + firstVal;
  }

  if (Array.isArray(firstVal) && Array.isArray(secondVal)) {
    return [].concat(secondVal, firstVal);
  }

  return (0, _merge.default)(secondVal, firstVal);
}

var _default = prepend;
exports.default = _default;
module.exports = exports.default;
});

var propAt_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Looks for a specified key on an Object you provide.
 * The is performed safely and will not throw an error if something on the prop path chain you specify doesn't exist.
 * Will always return `undefined` if a prop path cannot be resolved (rather than throwing).
 *
 * @function
 * @param {String|Array<String>} prop A top-level key OR a deeply nested prop path (which may be represented as an array or as a single dot-delimited string)
 * @param {object} obj An object which may contain a specified prop
 * @returns {*|undefined} The value associated with the nested prop path OR undefined if it does not exist
 */
function propAt(prop, obj) {
  try {
    var paths = typeof prop === "string" ? prop.split(".") : prop;
    var len = paths.length;
    if (len === 0) return undefined;
    var idx = 0;
    var val = obj[paths[idx]];

    while (++idx < len) {
      val = val[paths[idx]];
    }

    return val;
  } catch (err) {// Prop wasn't found; returning undefined
  }
}

var _default = propAt;
exports.default = _default;
module.exports = exports.default;
});

var propEquals_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _propAt = _interopRequireDefault(propAt_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Looks for a specified key on an Object you provide and checks to see if its corresponding value equals the value you specifiy.
 *
 * @function
 * @param {String} prop A key to search for on the Object
 * @param {Function|String} val A value that the extracted prop will be compared against
 * @param {object} obj An object which may contain a specified prop
 * @returns {Boolean} Whether or not the requested prop equals the specified value
 */
function propEquals(prop, val, obj) {
  return (0, _propAt.default)(prop, obj) === val;
}

var _default = propEquals;
exports.default = _default;
module.exports = exports.default;
});

var is_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _getConstructorName = _interopRequireDefault(getConstructorName_1);

var _getType = _interopRequireDefault(getType_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks to see if a value is a certain type.
 * You may specify that type as a case-insensitive string
 * (ie, 'string', 'boolean', 'object', 'function', 'array', 'regexp', 'date', 'set, 'map'), OR a JavaScript type constructor function (ie, String, Function, Boolean, Array, RegExp, Date, Set, Map).
 *
 * @function
 * @param {Function|String} ofType A JavaScript type constructor function (like `Function`, `String`, `RegExp`, `Boolean`, `Array`, `Object`, etc.) or a string value matching the name of one
 * @param {*} val A value (of any type)
 * @returns {Boolean} Whether or not the value matches the specified type
 * @example
 * is('boolean', true)
 * is('array', [1, 2, 3])
 * is(RegExp, /[a-z0-9]/)
 * is(Function, () => null)
 *
 */
function is(ofType, val) {
  return ofType === val || typeof ofType === "string" && (0, _getType.default)(val).toLowerCase() === ofType.toLowerCase() || ofType && !ofType.name && (0, _getType.default)(ofType) === (0, _getType.default)(val) || ofType && ofType.name && (0, _getConstructorName.default)(ofType) === "Function" && (0, _getType.default)(val) === ofType.name;
}

var _default = is;
exports.default = _default;
module.exports = exports.default;
});

var propIs_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _propAt = _interopRequireDefault(propAt_1);

var _is = _interopRequireDefault(is_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Looks for a specified key on an Object you provide and checks to see if its corresponding value is of the type you specifiy.
 *
 * @function
 * @param {Function|String} type A JavaScript type constructor function (ie `Boolean`, `RegExp`, `Date`, `Array`, `Object`, `Number`, `String`, etc) OR a string represention of the type (ie, "boolean", "regexp", "date", "array", "object", "number", "string", etc)
 * @param {String} prop A key to search for on the Object
 * @param {object} obj An object which may contain a specified prop
 * @returns {Boolean} Whether or not the requested prop is of the type specified
 */
function propIs(type, prop, obj) {
  return (0, _is.default)(type, (0, _propAt.default)(prop, obj));
}

var _default = propIs;
exports.default = _default;
module.exports = exports.default;
});

var propOr_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _propAt = _interopRequireDefault(propAt_1);

var _isUndefined = _interopRequireDefault(isUndefined_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Attempts to find a specified key on an Object you provide, and if not found will fall back to an additional value you specify.
 *
 * @function
 * @param {*} fallback A value to fall back on if the requested key does not exist on the provided Object
 * @param {String} prop A key to search for on the Object
 * @param {object} obj An object which may contain a specified prop
 * @returns {*} Either the requested prop (from the Object) or the fallback value
 */
function propOr(fallback, prop, obj) {
  var val = (0, _propAt.default)(prop, obj);
  return (0, _isUndefined.default)(val) ? fallback : val;
}

var _default = propOr;
exports.default = _default;
module.exports = exports.default;
});

var clone_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isNil = _interopRequireDefault(isNil_1);

var _isObject = _interopRequireDefault(isObject_1);

var _forIn = _interopRequireDefault(forIn_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Recursively copies the content of an Object into a new Object
 *
 * @function
 * @param {object|Array<*>} obj An Object (or Array) from which to create a deep copy
 * @returns {object|Array<*>} The new (cloned) Object (or Array)
 */
function clone(obj) {
  if (!(0, _isNil.default)(obj)) {
    if (Array.isArray(obj)) {
      return obj.map(function (v) {
        return (0, _isObject.default)(v) ? clone(v) : v;
      });
    }

    if ((0, _isObject.default)(obj)) {
      var newObj = {};
      (0, _forIn.default)(function (key, val) {
        if ((0, _isObject.default)(val) || Array.isArray(val)) {
          newObj[key] = clone(val);
        } else {
          newObj[key] = val;
        }
      }, obj);
      return newObj;
    }
  }
}

var _default = clone;
exports.default = _default;
module.exports = exports.default;
});

var propSet_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _has = _interopRequireDefault(has_1);

var _isObject = _interopRequireDefault(isObject_1);

var _clone = _interopRequireDefault(clone_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Looks for a specified key on an Object you provide and sets it to the provided value.
 * If the path does not exist, it will be created (you can check for the path via `propIs` or `propAt` or `propEquals` first if you don't wish to create the path every time).
 * The is performed safely and will not throw an error if somethign on the prop path chain you specify doesn't exist.
 *
 * @function
 * @param {String|Array<String>} prop A top-level key OR a deeply nested prop path (which may be represented as an array or as a single dot-delimited string)
 * @param {*} val A value to be placed at the provided property path
 * @param {object} obj An object which onto which the value will be placed
 * @returns {object} The original object, but modified to have the provided value placed at the specified path it does not exist
 */
function propSet(prop, val, obj) {
  var paths = typeof prop === "string" ? prop.split(".") : prop;
  var len = paths.length;
  if (len === 0) return obj;
  var newObj = (0, _clone.default)(obj);

  if (len > 1) {
    newObj[paths[0]] = propSet(paths.slice(1), val, !(0, _has.default)(paths[0], newObj) || !(0, _isObject.default)(newObj[paths[0]]) ? {} : newObj[paths[0]]);
  } else {
    newObj[paths[0]] = val;
  }

  return newObj;
}

var _default = propSet;
exports.default = _default;
module.exports = exports.default;
});

var reduce_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * A simple wrapper around native Array.prototype.reduce(), for use in a compose/pipe chain of functions.
 * Can also reduce the key/value pairs of an object, if one is supplied in place of an array.
 *
 * @function
 * @param {Function} fn A function to control the reduction of each item in the array|object into the single output value
 * @param {*} defaultVal A starting value for the reduction accumulator
 * @param {Array<*>|object} arr An array of values of any type OR an object containing key/value pairs
 * @returns {*} The original Array|Object somehow reduced to one value, according to the supplied function
 */
function reduce(fn, defaultVal, arr) {
  return Array.isArray(arr) ? arr.reduce(fn, defaultVal) : Object.keys(arr).reduce(function (acc, key) {
    return fn(acc, arr[key], key);
  }, defaultVal);
}

var _default = reduce;
exports.default = _default;
module.exports = exports.default;
});

var renameKeys_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _forIn = _interopRequireDefault(forIn_1);

var _has = _interopRequireDefault(has_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Renames a set of keys in a given object (removing the old ones)
 *
 * @function
 * @param {object} keyMap An object whose keys are the _current_ key names and whose values are the _new_ key names
 * @param {object} obj An Object whose keys will be renamed
 * @returns {object} A new Object that has all the specified keys renamed to their new names
 */
function renameKeys(keyMap, obj) {
  var newObj = {};
  (0, _forIn.default)(function (key, val) {
    if ((0, _has.default)(key, keyMap)) {
      newObj[keyMap[key]] = val;
    } else {
      newObj[key] = val;
    }
  }, obj);
  return newObj;
}

var _default = renameKeys;
exports.default = _default;
module.exports = exports.default;
});

var coerce = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.toArray = exports.toFunction = void 0;

var toFunction = function toFunction(v) {
  return typeof v === "function" ? v : function () {
    return v;
  };
};

exports.toFunction = toFunction;

var toArray = function toArray(v) {
  return Array.isArray(v) ? v : [v];
};

exports.toArray = toArray;
});

var validate_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _cond = _interopRequireDefault(cond_1);

var _forIn = _interopRequireDefault(forIn_1);

var _mapObject = _interopRequireDefault(mapObject_1);



function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isFunction = function isFunction(v) {
  return typeof v === "function";
};

var brokenValidator = function brokenValidator(val, key) {
  return "Validations for \"" + key + "\" are broken";
};

var defaultMessageHandler = function defaultMessageHandler(val, key) {
  return "\"" + val + "\" is not valid for \"" + key + "\"";
};

function _unpackValidator(rawValidators) {
  var validators = (0, _cond.default)([[isFunction, [[rawValidators, defaultMessageHandler]]], [function (v) {
    return Array.isArray(v) && v.length <= 2 && isFunction(v[0]);
  }, function (_ref) {
    var validator = _ref[0],
        _ref$ = _ref[1],
        message = _ref$ === void 0 ? defaultMessageHandler : _ref$;
    return [[validator, (0, coerce.toFunction)(message)]];
  }], [function (v) {
    return Array.isArray(v) && v.every(function (kv) {
      return Array.isArray(kv);
    });
  }, function (v) {
    return v.map(function (_ref2) {
      var validator = _ref2[0],
          _ref2$ = _ref2[1],
          message = _ref2$ === void 0 ? defaultMessageHandler : _ref2$;
      return [validator, (0, coerce.toFunction)(message)];
    });
  }], [function (v) {
    return Array.isArray(v);
  }, function (v) {
    return v.map(function (validator) {
      if (Array.isArray(validator) && isFunction(validator[0])) {
        return [validator[0], (0, coerce.toFunction)(validator[1] || defaultMessageHandler)];
      } else if (isFunction(validator)) {
        return [validator, defaultMessageHandler];
      }

      return [function () {
        return true;
      }, brokenValidator];
    });
  }], [function () {
    return true;
  }, [[function () {
    return false;
  }, brokenValidator]]]], rawValidators);
  return function (v, k) {
    return validators.map(function (_ref3) {
      var validator = _ref3[0],
          createErrorMessage = _ref3[1];
      return validator(v, k) || createErrorMessage(v, k);
    }).filter(function (kv) {
      return kv !== true;
    });
  };
}
/**
 * Applies many validations to an object of values.
 * Those validations are written as Arrays of pairs:
 *   - for the value at the first index supply your validation function that _must_ pass
 *   - for the value at the second index supply a corresponding error message (OR function returning an error message)
 *     should the validation fail.
 *
 * If you do not supply a validation error message, a default one will be built that looks like:
 *   `"${val}" is not valid for "${key}"`
 *
 * And of course, you can write your validation error message as a function that will receive the value and key name as its first and second params (you should return a message as a string).
 *
 * If you do not supply valid input (Array of validator function and validation error message), rather than throwing an error it will be replaced with a validation function that always returns `false` and a corresponding error message of:
 *   `Validations for "${key}" are broken`
 *
 * @function
 * @param {Array<Function>} validations An array of validator functions and their corresponding error message
 * @param {object} values An Object of key value pairs, the keys should correspond to validators and the values are that which is to be validated
 * @returns {object} An object containing the key names of the values and one or more validation error messages (Only key names whose values were found to be invalid will show up on this output Object; an empty Object means everything was valid)
 * @example
 * validate(isRequired, val)
 * validate([isRequired, 'field is required'], val)
 * validate([
 *   [isRequired, (_, key) => `${key} is required`],
 *   [isValidThing, val => `${val} is not a valid thing`],
 *   [mustBeBlue, 'Your favorite color must be blue'],
 *   [functionalOrObjectOriented]
 * ], val)
 */


function validate(validations, values) {
  var validators = (0, _mapObject.default)(_unpackValidator, validations);
  var invalids = {};
  (0, _forIn.default)(function (key, val) {
    var validationResults = val(values[key], key);

    if (validationResults.length) {
      invalids[key] = validationResults;
    }
  }, validators);
  return invalids;
}

var _default = validate;
exports.default = _default;
module.exports = exports.default;
});

var eitherOr_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _curry = _interopRequireDefault(curry_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * A function that accepts two functions and a value and will return the first result which is "truthy".
 * This function is curried because it doesn't make any sense to even have it unless you have the functions ahead of time but not the value.
 *
 * @function
 * @param {function} fnA The first function to be executed on the value
 * @param {function} fnB The second function to be executed on the value
 * @param {*} val A value to be passed into the functions
 * @returns {*} The result of executing value passed in
 */
function eitherOr(fnA, fnB, val) {
  if (typeof fnA !== "function") {
    throw new TypeError(fnA + " is not a function");
  }

  if (typeof fnB !== "function") {
    throw new TypeError(fnB + " is not a function");
  }

  return fnA(val) || fnB(val);
}

var _default = (0, _curry.default)(eitherOr);

exports.default = _default;
module.exports = exports.default;
});

var curried = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.validate = exports.merge = exports.renameKeys = exports.reduce = exports.propSet = exports.propOr = exports.propIs = exports.propEquals = exports.propAt = exports.prepend = exports.pick = exports.omit = exports.mapSpec = exports.mapString = exports.mapObjectRecursive = exports.mapObject = exports.map = exports.isStrictEqual = exports.isSameType = exports.isEqual = exports.intersection = exports.hasNestedProp = exports.has = exports.fuzzy = exports.filterString = exports.filterObject = exports.filter = exports.findIndex = exports.find = exports.each = exports.difference = exports.contains = exports.cond = exports.append = exports.eitherOr = void 0;

var _curry = _interopRequireDefault(curry_1);

var _append = _interopRequireDefault(append_1);

var _cond = _interopRequireDefault(cond_1);

var _contains = _interopRequireDefault(contains_1);

var _difference = _interopRequireDefault(difference_1);

var _each = _interopRequireDefault(each_1);

var _find = _interopRequireDefault(find_1);

var _findIndex = _interopRequireDefault(findIndex_1);

var _filter = _interopRequireDefault(filter_1);

var _filterObject = _interopRequireDefault(filterObject_1);

var _filterString = _interopRequireDefault(filterString_1);

var _fuzzy = _interopRequireDefault(fuzzy_1);

var _has = _interopRequireDefault(has_1);

var _hasNestedProp = _interopRequireDefault(hasNestedProp_1);

var _intersection = _interopRequireDefault(intersection_1);

var _isEqual = _interopRequireDefault(isEqual_1);

var _isSameType = _interopRequireDefault(isSameType_1);

var _isStrictEqual = _interopRequireDefault(isStrictEqual_1);

var _map = _interopRequireDefault(map_1);

var _mapObjectRecursive = _interopRequireDefault(mapObjectRecursive_1);

var _mapObject = _interopRequireDefault(mapObject_1);

var _mapSpec = _interopRequireDefault(mapSpec_1);

var _mapString = _interopRequireDefault(mapString_1);

var _omit = _interopRequireDefault(omit_1);

var _pick = _interopRequireDefault(pick_1);

var _prepend = _interopRequireDefault(prepend_1);

var _propAt = _interopRequireDefault(propAt_1);

var _propEquals = _interopRequireDefault(propEquals_1);

var _propIs = _interopRequireDefault(propIs_1);

var _propOr = _interopRequireDefault(propOr_1);

var _propSet = _interopRequireDefault(propSet_1);

var _reduce = _interopRequireDefault(reduce_1);

var _renameKeys = _interopRequireDefault(renameKeys_1);

var _merge = _interopRequireDefault(merge_1);

var _validate = _interopRequireDefault(validate_1);

var _eitherOr = _interopRequireDefault(eitherOr_1);

exports.eitherOr = _eitherOr.default;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var append = (0, _curry.default)(_append.default);
exports.append = append;
var cond = (0, _curry.default)(_cond.default);
exports.cond = cond;
var contains = (0, _curry.default)(_contains.default);
exports.contains = contains;
var difference = (0, _curry.default)(_difference.default);
exports.difference = difference;
var each = (0, _curry.default)(_each.default);
exports.each = each;
var find = (0, _curry.default)(_find.default);
exports.find = find;
var findIndex = (0, _curry.default)(_findIndex.default);
exports.findIndex = findIndex;
var filter = (0, _curry.default)(_filter.default);
exports.filter = filter;
var filterObject = (0, _curry.default)(_filterObject.default);
exports.filterObject = filterObject;
var filterString = (0, _curry.default)(_filterString.default);
exports.filterString = filterString;
var fuzzy = (0, _curry.default)(_fuzzy.default);
exports.fuzzy = fuzzy;
var has = (0, _curry.default)(_has.default);
exports.has = has;
var hasNestedProp = (0, _curry.default)(_hasNestedProp.default);
exports.hasNestedProp = hasNestedProp;
var intersection = (0, _curry.default)(_intersection.default);
exports.intersection = intersection;
var isEqual = (0, _curry.default)(_isEqual.default);
exports.isEqual = isEqual;
var isSameType = (0, _curry.default)(_isSameType.default);
exports.isSameType = isSameType;
var isStrictEqual = (0, _curry.default)(_isStrictEqual.default);
exports.isStrictEqual = isStrictEqual;
var map = (0, _curry.default)(_map.default);
exports.map = map;
var mapObject = (0, _curry.default)(_mapObject.default);
exports.mapObject = mapObject;
var mapObjectRecursive = (0, _curry.default)(_mapObjectRecursive.default);
exports.mapObjectRecursive = mapObjectRecursive;
var mapString = (0, _curry.default)(_mapString.default);
exports.mapString = mapString;
var mapSpec = (0, _curry.default)(_mapSpec.default);
exports.mapSpec = mapSpec;
var omit = (0, _curry.default)(_omit.default);
exports.omit = omit;
var pick = (0, _curry.default)(_pick.default);
exports.pick = pick;
var prepend = (0, _curry.default)(_prepend.default);
exports.prepend = prepend;
var propAt = (0, _curry.default)(_propAt.default);
exports.propAt = propAt;
var propEquals = (0, _curry.default)(_propEquals.default);
exports.propEquals = propEquals;
var propIs = (0, _curry.default)(_propIs.default);
exports.propIs = propIs;
var propOr = (0, _curry.default)(_propOr.default);
exports.propOr = propOr;
var propSet = (0, _curry.default)(_propSet.default);
exports.propSet = propSet;
var reduce = (0, _curry.default)(_reduce.default);
exports.reduce = reduce;
var renameKeys = (0, _curry.default)(_renameKeys.default);
exports.renameKeys = renameKeys;
var merge = (0, _curry.default)(_merge.default);
exports.merge = merge;
var validate = (0, _curry.default)(_validate.default);
exports.validate = validate;
});

var isEmpty_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isObject = _interopRequireDefault(isObject_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks if a value is empty. Arrays, Objects, Strings, Sets, and Null/Undefined values are considered empty if their length (or size) prop is zero (or if they are Null or Undefined).
 * Whitespace-only strings are NOT considered empty (use `isBlankString` instead).
 *
 * @function
 * @param {*} val A value of any type which may be considered empty
 * @returns {Boolean} Whether or not the value is empty
 */
function isEmpty(val) {
  return val == null || val.length === 0 || val.size === 0 || (0, _isObject.default)(val) && Object.keys(val).length === 0;
}

var _default = isEmpty;
exports.default = _default;
module.exports = exports.default;
});

var isEmpty = /*@__PURE__*/getDefaultExportFromCjs(isEmpty_1);

var areAllFunctions_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

function areAllFunctions(fns) {
  return fns.some(function (f, idx) {
    if (typeof f !== "function") {
      throw new Error("Argument #" + (idx + 1) + " passed to \"compose()\" is not a function:\n\"" + JSON.stringify(f) + "\"");
    }

    return true;
  });
}

var _default = areAllFunctions;
exports.default = _default;
module.exports = exports.default;
});

var identity_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * A function that always returns the value passed to it
 *
 * @function
 * @param {*} v A value to be returned
 * @returns {*} The value passed in
 */
function identity(v) {
  return v;
}

var _default = identity;
exports.default = _default;
module.exports = exports.default;
});

var pipe_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _areAllFunctions = _interopRequireDefault(areAllFunctions_1);

var _identity = _interopRequireDefault(identity_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Creates a chain of Functions that will be executed in sequnce (from left to right), with the value from the previous Function fed into the next Function.
 * The value that the chain of functions will executed on can be provided later.
 *
 * @function
 * @param {Function} ...fns One or more function to execute (in sequential order) on a value that will be supplied later
 * @returns {Function} A single Function that is ready to receive a value and pass it through the piped chain of Functions
 */
function pipe() {
  for (var _len = arguments.length, fns = new Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  var fnlen = fns.length;
  if (fnlen === 0 || !(0, _areAllFunctions.default)(fns)) return _identity.default;

  function inner() {
    var result = fns[0].apply(fns, arguments);

    for (var i = 1; i < fnlen; i++) {
      result = fns[i](result);
    }

    return result;
  }

  return inner;
}

var _default = pipe;
exports.default = _default;
module.exports = exports.default;
});

var pipe = /*@__PURE__*/getDefaultExportFromCjs(pipe_1);

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

/**
 * Turns any non-Object values into empty objects
 *
 * @function
 * @name objectify
 * @sig * -> {k: v}
 * @param {*} val A value of any type
 * @returns {Object} If the original value was an Object, it is returned as-is, otherwise an empty object is returned
 */

var objectify = function objectify(val) {
  return isObject(val) ? val : {};
};
/**
 * If the provided value is NOT a function, transform it into a function that always returns that value
 *
 * @function
 * @name alwaysFunction
 * @sig * -> (* -> *)
 * @param {*} val A value of any type
 * @returns {Function} If the original value was a function, that function is returned, otherwise a function that always returns the original value is returned function that always returns that value OR
 */

var alwaysFunction = function alwaysFunction(val) {
  return typeof val === 'function' ? val : function () {
    return val;
  };
};
/**
 * Makes sure a given spec is acceptable for one of the pruning modes (ie, "remove" or "keep").
 * Specs in this mode are treated more strictly, meaning the prop must be given a value of "true" or the key and value can be identical.
 * Functions are acceptable values in the spec of course.
 *
 * @function
 * @name makePruningSpec
 * @sig {k: v} -> {k: v}
 * @param {Object} spec A spec to be coerced into an acceptable pruning spec
 * @returns {Object} A spec that is acceptable to be used in pruning mode.
 */

var makePruningSpec = function makePruningSpec(val) {
  var obj = objectify(val);
  return Object.fromEntries(Object.entries(obj).filter(function (_ref) {
    var key = _ref[0],
        v = _ref[1];
    return key === v || v === true || typeof v === 'function';
  }));
};
/**
 * Filters a spec (object) to only the props that are transform functions.
 * If the input is not an object, this is just an identity function.
 *
 * @function
 * @name onlySpecTransforms
 * @sig {k: v} -> {k: v}
 * @param {Object} spec An object whose values (may) be transform functions
 * @returns {Object} The input object with only the props that are functions retained
 */

var onlySpecTransforms = function onlySpecTransforms(val) {
  if (!isObject(val)) {
    return val;
  }

  return Object.fromEntries(Object.entries(val).filter(function (_ref2) {
    var key = _ref2[0],
        v = _ref2[1];
    return key === 'shapeyDebug' || typeof v === 'function';
  }));
};
/**
 * Removes the reserved "shapey*" prefixed props from a spec.
 *
 * @function
 * @name removeMagicProps
 * @sig {k: v} -> {k: v}
 * @param {Object} spec A shapey spec to be pruned of magic props
 * @returns {Object} A shapey spec cleaned of magic props
 */

var removeMagicProps = function removeMagicProps(val) {
  return Object.fromEntries(Object.entries(val).filter(function (_ref3) {
    var key = _ref3[0];
    return !/^shapey/i.test(key);
  }));
};
/**
 * Takes an non-transform props (non-functions) in a given spec and merges them onto the input object.
 *
 * @function
 * @name applyNonTransformProps
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with only the pass-through props applied to it
 */

var applyNonTransformProps = curry(function (spec, value) {
  return [spec, value].every(function (v) {
    return isObject(v);
  }) ? merge(value, filter(function (v) {
    return typeof v !== 'function';
  }, removeMagicProps(spec))) : value;
});
/**
 * Logs to the console any failed transform functions, along with the field name and the value that was fed into the transform function.
 * Plus, the exception itself is also logged.
 *
 * @function
 * @name defaultErrorHandler
 * @sig String -> * -> Error -> *
 * @param {String} fieldName The field name for the failed transform function
 * @param {Object} value The value that was fed into the transform function
 * @param {Object} exception The exception thrown by the failed transform
 */

var defaultErrorHandler = curry(function (fieldName, value, exception) {
  return (// eslint-disable-next-line no-console
    console.error("\n Transform failed on field: \"" + fieldName + "\"\n value:", value, '\n', exception)
  );
});
/**
 * A wrapper around an (optional) error handler that the may be passed in as the "shapeyDebug" value.
 * It is curried, but will feed their handler a more standard signature of: (err, field, value)
 *
 * @function
 * @name wrapperForTheirErrorHandler
 * @sig ((Error, String, *) -> *) -> String -> * -> Error -> *
 * @param {Function} errHandler The custom error handler provided by the consumer
 * @param {String} fieldName The field name for the failed transform function
 * @param {Object} value The value that was fed into the transform function
 * @param {Object} exception The exception thrown by the failed transform
 * @returns {*} Could be anything, or nothing (it's up to the consumer)
 */

var wrapperForTheirErrorHandler = curry(function (errHandler, fieldName, value, exception) {
  return errHandler(exception, fieldName, value);
});
/**
 * Wraps every function on a given spec in a try/catch that catches any exception.
 *
 * What it _does_ with the exception is up to the consumer:
 *   - ignores it (if "shapeyDebug" isn't set)
 *   - logs it along with the corresponding field name,
 *     using `console.error` (if "shapeyDebug" is set to `true`)
 *   - uses a custom handler supplied by the consumer as the value for "shapeyDebug"
 *
 * Note that a custom error handler will be passed the following params (in order):
 *   - The exception
 *   - The field name
 *   - The value that was fed into the transform function
 *
 * @function
 * @name safeSpecTransforms
 * @sig {k: (a -> b)} -> {k: (a -> b)}
 * @param {Object} spec An object whose values may be transform functions that need to be wrapped in try/catch
 * @returns {Object} The same spec object but whose functions are safely wrapped in in try/catch handlers
 */

function safeSpecTransforms(spec) {
  var handleError = function handleError() {
    return function () {
      return undefined;
    };
  };

  var _objectify = objectify(spec),
      shapeyDebug = _objectify.shapeyDebug,
      obj = _objectWithoutPropertiesLoose(_objectify, ["shapeyDebug"]);

  if (shapeyDebug === true) {
    handleError = defaultErrorHandler;
  } else if (typeof shapeyDebug === 'function') {
    handleError = wrapperForTheirErrorHandler(shapeyDebug);
  } else if (/^skip$/i.test(shapeyDebug)) {
    handleError = function handleError(_, originalValue) {
      return function () {
        return originalValue;
      };
    };
  }

  return mapObject(function (transform, fieldName) {
    if (typeof transform !== 'function') {
      return transform;
    }

    return curry(function (fn, val) {
      try {
        return fn(val);
      } catch (err) {
        return handleError(fieldName, val)(err);
      }
    })(transform);
  }, obj);
}

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but any primitive value or array in the transform spec is wrapped in a function that always returns that value.
 * Any functions already on the transform spec are applied at the prop-level (like evolve() always does) and any objects on the transform spec call the evolver recursively (again, normal behavior for evolve()).
 * In other words, the only difference between this and Ramda's evolve() function is that primitives and array vales are wrapped in an always() function before the spec is fed to evolve() rather than being omitted from the final output.
 *
 * @function
 * @name evolveSpec
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object with transform functions or nested transform functions or primitive vales to be wrapped with a function.
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms applied to it
 */

var evolveSpec = curry(function (transforms, input) {
  var result = {};
  var spec = safeSpecTransforms(transforms);
  var object = objectify(input);
  Object.keys(object).forEach(function (key) {
    if (typeof spec[key] === 'function') {
      result[key] = spec[key](object[key]);
    } else if (isObject(spec[key])) {
      result[key] = evolveSpec(spec[key], object[key]);
    } else if (has(key, spec)) {
      result[key] = spec[key];
    } else {
      result[key] = object[key];
    }
  });
  return result;
});

/**
 * Keeps only the props you name in your spec.
 * It is expected (best practice, for conveying intent) that you give a value of `true` for props that you want to keep
 * but _not_ transform (a zipped object - where key and value are both strings of the same name - makes sense too).
 *
 * @function
 * @name keeper
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are `true` or match the key name to signal a value to be preserved (as-is) on the output
 * @param {Object} input An object which will be passed through the pruning defined by the spec
 * @returns {Object} The input object with the unnamed props shaved off the output
 */

var keeper = curry(function (spec, input) {
  return pipe(objectify, pipe(makePruningSpec, Object.keys, curried.pick)(spec))(input);
});
/**
 * Keeps only the props you name in your spec. If you provided a function, it is applied at the prop-level.
 * It is expected (best practice, for conveying intent) that you give a value of `true` for props that you want to keep
 * but _not_ transform (a zipped object - where key and value are both strings of the same name - makes sense too).
 * In this mode, if you provide any other type of value it won't be used in the spec.
 * If the prop _doesn't_ exist, you won't see it on the output
 * (use shapeSuperStrictly if you want to see `undefined` for those cases instead).
 *
 * @function
 * @name keepAndShape
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or (preferrably) a value of `true` to signal a value to be preserved (as-is) on the output
 * @param {Object} input An object which will be passed through the pruning/re-shaping defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it and unnamed props shaved off the output
 */

var keepAndShape = curry(function (spec, input) {
  return pipe(keeper(spec), evolveSpec(curried.filter(function (v) {
    return typeof v === 'function';
  }, spec)))(input);
});
/**
 * Removes the props you name in your spec.
 * It is expected (best practice, for conveying intent) that you give a value of `true` for props that you want to remove
 * but _not_ transform (a zipped object - where key and value are both strings of the same name - makes sense too).
 *
 * @function
 * @name remover
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are `true` or match the key name to signal a value to be removed from the output
 * @param {Object} input An object which will be passed through the pruning defined by the spec
 * @returns {Object} The input object with the named props shaved off the output
 */

var remover = curry(function (spec, input) {
  return pipe(objectify, pipe(makePruningSpec, curried.filter(function (v) {
    return typeof v !== 'function';
  }), Object.keys, curried.omit)(spec))(input);
});
/**
 * Removes all the props you name in your spec. If you provided a transform function however,
 * it is applied at the prop-level and that prop is (obviously) NOT removed but rather transformed accordingly.
 * It is expected (best practice, for conveying intent) that you give a value of `true` for props you want to remove
 * (a zipped object - where key and value are both strings of the same name - makes sense too).
 * In this mode, if you provide any other type of value it won't be used in the spec.
 *
 * @function
 * @name removeAndShape
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or (preferrably) a value of `true` to signal a value to be remove from the output
 * @param {Object} input An object which will be passed through the pruning/re-shaping defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it and name props shaved off the output
 */

var removeAndShape = curry(function (spec, input) {
  return pipe(remover(spec), evolveSpec(curried.filter(function (v) {
    return typeof v === 'function';
  }, spec)))(input);
});
/**
 * Removes the any props from the input not named in the spec. Doesn't matter
 * what the value types are in the spec: if the key name doesn't exist in the
 * spec, it won't exist in the output.
 *
 * @function
 * @name impliedRemove
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object
 * @param {Object} input An object which will have any props not named in the spect removed
 * @returns {Object} The input object with the props removed which were not in the spec too
 */

var impliedRemove = curry(function (spec, input) {
  return curried.pick(Object.keys(spec), input);
});

/**
 * A port of [Ramda's evolve()](http://ramdajs.com/docs/#evolve), but the transforms are _always_ applied regardless if the key/value pair exists in the input object or not.
 *
 * @function
 * @name alwaysEvolve
 * @sig {k: (a -> b)} -> {k: v} -> {k: v}
 * @param {Object} transforms An object whose values may be transform functions that need to be wrapped in try/catch
 * @param {Object} input The input object to pass through prop-level transformations
 * @returns {Object} The modified input object, with any prop-level transforms applied to it
 */

var alwaysEvolve = curry(function (transforms, input) {
  var result = {};
  var spec = safeSpecTransforms(transforms);
  var object = objectify(input);
  Object.keys(spec).forEach(function (key) {
    if (typeof spec[key] === 'function') {
      result[key] = spec[key](object[key]);
    } else if (isObject(spec[key])) {
      result[key] = alwaysEvolve(spec[key], object[key]);
    } else {
      result[key] = spec[key];
    }
  });
  return merge(input, result);
});

var curryN_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

/**
 * Same as the regular curry function, but you must also specify the number of arguments that the curried function will take before being fully executed.
 * You would use this in cases where it cannot be inferred from the curried function itself.
 *
 * @function
 * @throws {TypeError} if arity is not supplied or is not a positive number
 * @param {Number} arity The number of arguments the curried function will received before being executed
 * @param {Function} fn A Function whose signature needs to changed from requiring all at once to providing them one (or more) at a time.
 * @returns {Function} A new Function that will wait until all (arity) arguments have been supplied before returning a result (otherwise it will continue to return a new Function that is ready to receive the next argument)
 */
function curryN(arity, fn) {
  if (typeof arity !== "number" || Number.isNaN(arity) || arity <= 0) {
    return fn;
  }

  var fnlen = arity;
  return function manyArgCurry() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var arglen = args.length;

    if (arglen === fnlen) {
      return fn.apply(void 0, args);
    }

    return function inner() {
      for (var _len2 = arguments.length, innerArgs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        innerArgs[_key2] = arguments[_key2];
      }

      var innerlen = innerArgs.length;
      var limit = fnlen;
      var allArgs = [];
      var argCount = 0;

      while (argCount < arglen) {
        allArgs.push(args[argCount]);
        argCount++;
        limit--;
      }

      var innerArgCount = 0;

      while (innerArgCount < innerlen) {
        allArgs.push(innerArgs[innerArgCount]);
        innerArgCount++;
        limit--;
      }

      return limit > 0 ? manyArgCurry.apply(void 0, allArgs) : fn.apply(void 0, allArgs);
    };
  };
}

var _default = curryN;
exports.default = _default;
module.exports = exports.default;
});

var curryN = /*@__PURE__*/getDefaultExportFromCjs(curryN_1);

var makeSpec = function makeSpec(transforms) {
  var spec = map(function (v) {
    return isObject(v) ? makeSpec(v) : alwaysFunction(v);
  }, safeSpecTransforms(transforms));
  var arity = Math.max.apply(Math, Object.values(spec).filter(function (v) {
    return has('length', v);
  }).map(function (v) {
    return v.length;
  }).concat([0]));
  return curryN(arity, function () {
    for (var _len = arguments.length, allArgs = new Array(_len), _key = 0; _key < _len; _key++) {
      allArgs[_key] = arguments[_key];
    }

    return map(function (fn) {
      return fn.apply(void 0, allArgs);
    }, spec);
  });
};
/**
 * A port over of [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec), but with a few changes you may or may not see as necessary.
 * (1) Properly curried so that you can optionally provide args as (spec, input) rather than only as (spec)(input)
 * (2) It wraps each function in a tryCatch that logs the key name of the transform function that fails.
 *
 * Using applySpec() in the wild and at-scale proved to be one of the hardest Ramda functions to debug.
 * Limiting its functionality and adding console logging doesn't sound like a necessary thing, but when working with Ramda on a full team it may help prevent devs from getting so frustrated that they vote to abandon the library while they're still learning the functional programming paradigm.
 *
 * @function
 * @name mapSpec
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which expect the entire object to be fed in as input.  This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object transformed to contain only the props named in the spec (with the corresponding transforms applied at that key)
 */


var mapSpec = curry(function (transforms, input) {
  return makeSpec(transforms)(input);
});

/**
 * Applies transform functions in a given spec to a given object, BUT only the
 * trasnforms that DON'T match key names on the input object are applied.
 *
 * @function
 * @name applyWholeObjectTransforms
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object which contains transform functions (only those which do not correspond to keys on the input object will be applied to the input object)
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it (only those that do not correspond to key names on the input object)
 */

var applyWholeObjectTransforms = curry(function (spec, value) {
  var nonFunctionPropNames = difference(Object.keys(curried.filter(function (v) {
    return typeof v === 'function';
  }, spec)), Object.keys(value));
  var nonFunctionProps = pick(nonFunctionPropNames, spec);
  return mapSpec(nonFunctionProps);
});
var baseShaper = curry(function (evolver, spec, value) {
  var baseObject = applyNonTransformProps(spec)(value);
  var baseTransformedObject = evolver(onlySpecTransforms(spec))(baseObject);
  var transformFn = applyWholeObjectTransforms(spec)(baseTransformedObject);
  return merge(baseTransformedObject, transformFn(isEmpty(baseTransformedObject) ? value : baseTransformedObject));
});
/**
 * This function allows one to blend an object with a clone that is taken through prop transformations. Unlike object mappers/transformers, not every value in the spec need be a function.
 * Non-function values are simply added to the result.
 * If a given value in the spec IS a function however, it will be used to transform OR to create a prop.
 * If the prop specified on the spec exists on the value (object) being mapped, a prop-level transform will be perform (just like [Ramda's evolve()](http://ramdajs.com/docs/#evolve)).
 * But if a prop in the spec doesn't exist in the object and if that prop is a transformer function, then the transformer will be used to create that prop on the resulting object BUT the entire object will be passed into the transform function (just like [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec), rather than just a single prop.
 *
 * @function
 * @name shapeLoosely
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */

var shapeLoosely = baseShaper(evolveSpec);
/**
 * Applies a shaping spec to an input object, but will NOT pass through any props unless they are named in the spec.
 * In other words, you're providing it template for creating a brand new object from some raw input.
 *
 * @function
 * @name shapeStrictly
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */

var shapeStrictly = curry(function (spec, input) {
  var reshapedObj = shapeLoosely(spec, input);
  return pick(Object.keys(spec), reshapedObj);
});
var pruneOutput = curry(function (spec, input) {
  if (!isObject(input)) {
    return input;
  }

  if (typeof spec === 'function') {
    return spec(input);
  } else if (isObject(spec)) {
    switch ((spec.shapeyMode || '').toLowerCase()) {
      case 'remove':
        return remover(spec, input);

      case 'keep':
        return keeper(spec, input);

      case 'strict':
        return applyNonTransformProps(spec)(impliedRemove(spec, input));

      default:
        return applyNonTransformProps(spec)(input);
    }
  } else {
    return spec;
  }
});
var applyTransforms = curry(function (spec, input) {
  if (typeof spec === 'function') {
    return spec(input);
  } else if (isObject(spec)) {
    switch ((spec.shapeyTransforms || '').toLowerCase()) {
      case 'prop':
        return alwaysEvolve(onlySpecTransforms(spec))(input);

      case 'whole':
        return mapSpec(onlySpecTransforms(spec))(input);

      default:
        return shapeLoosely(onlySpecTransforms(spec))(input);
    }
  } else {
    return spec;
  }
});
/**
 * A single function that selects and applies one of the available re-shaping functions in the shapey library,
 * depending on what you've set the `shapeyMode` prop to in the `spec` you provide as the first argument to this function.
 *
 * Think of it like a case statement in a Redux reducer, however since you most likely _don't_ want to sacrifice
 * the meanining associated with the "type" property to internals of shapey, a prop called "shapeyMode" is used instead.
 *
 * If for some reason you have some prop on the input (the object you're transforming) already named "shapeyMode", um . . . don't.
 *
 * Available modes (case & space in-sensitive):
 *   "strict" - uses `shapeStrictly`, where _only_ the props named in your spec are included in the output
 *   "keep" - uses `keepAndShape`, where all the props you name in your spec are kept.
 *   "remove" - uses `removeAndShape`, where all the props you name in your spec are removed.
 *
 * In addition to controlling the mode for Shapey, you can control how the transforms are applied.
 * This is controlled via the reserved prop in your spec called "shapeyTransforms", and the available options for it are:
 *   "prop" - All transforms are applied at the prop-level, regardless if they exist on the input object
 *   "whole" - All transforms are given the _entire_ input object as input
 *   (regardless if a prop matching the name of the transform exists on the input object)
 *
 * @function
 * @name makeShaper
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are either transform functions or pass-through values to be added to the return object
 * @param {Object} input An object which will be passed through the re-shaping transform functions defined by the spec
 * @returns {Object} The input object with the spec transforms applied to it
 */

var makeShaper = curry(function (spec, input) {
  var transformedObj = applyTransforms(spec, input);
  var prunedObj = pruneOutput(spec, transformedObj);
  return isObject(prunedObj) ? omit(['shapeyTransforms', 'shapeyMode'], prunedObj) : prunedObj;
});
/**
 * Applies a list of functions (in sequence) to a single input, passing
 * the transformed output as the input value to the next function in the chain.
 * If a spec object is included in the transforms, the shaper that corresponds
 * to the "shapeyMode" prop is invoked (otherwise `shapeLoosely()` is used).
 * This creates a transform function for the pipeline.
 *
 * @function
 * @name shapeline
 * @sig [a -> b, b -> c, c -> d, ...] -> * -> *
 * @param {Function[]} transforms A list of transform functions
 * @param {*} input The input value to pass through the enhancer pipeline
 * @returns {*} The output of the original input passed through the chain of transform functions
 */

var shapeline = curry(function (transforms, input) {
  return curried.reduce(function (inputObj, fn) {
    return fn(inputObj);
  }, input, curried.map(makeShaper, transforms || []));
});

var flatten_1 = createCommonjsModule(function (module, exports) {

exports.__esModule = true;
exports.default = void 0;

var _isArrayish = _interopRequireDefault(isArrayish_1);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Extracts nested arrays (of any depth) from a provided array, placing them onto on single new array.
 *
 * @function
 * @param {Array<Array<*>>|Array<*>} arr An array of values that may or may not be nested arrays themselves
 * @returns {Array<*>} A new array of values, but with any nested arrays from the original input extracted onto one single (flat) array
 */
function flatten(arr) {
  var newArr = [];
  var i = -1;
  var len = arr.length;

  while (++i < len) {
    var val = arr[i];

    if ((0, _isArrayish.default)(val)) {
      var j = -1;
      var nlen = val.length;

      while (++j < nlen) {
        newArr.push(val[j]);
      }
    } else {
      newArr.push(val);
    }
  }

  return newArr;
}

var _default = flatten;
exports.default = _default;
module.exports = exports.default;
});

var flatten = /*@__PURE__*/getDefaultExportFromCjs(flatten_1);

/**
 * Combines two values of the same type (if it makes sense to combine them).
 * Numbers are summarized, strings and arrays are concatenated together, and true objects are merged (the second value merged on top of the first).
 * In any other case only the first value is returned.
 *
 * @function
 * @name combine
 * @sig a -> a -> a
 * @param {*} val1 The base value to be combined with
 * @param {*} val2 The value to combine
 * @returns {*} If the values are of the same type, this represents the combined value of the two of them. Otherwise only the first value is returned
 */

var combine = curry(function (val1, val2) {
  if (typeof val1 === 'number' && typeof val2 === 'number') {
    return val1 + val2;
  } else if (typeof val1 === 'string' && typeof val2 === 'string') {
    return [val1, val2].join('');
  } else if (Array.isArray(val1) && Array.isArray(val2)) {
    return flatten([val1, val2]);
  } else if (isObject(val1) && isObject(val2)) {
    return merge(val1, val2);
  } else {
    return val1;
  }
});

/**
 * Similar to [Ramda's applySpec()](http://ramdajs.com/docs/#applySpec), however the spec will be applied and then merged into the original input.
 * Ramda's applySpec() will omit the original input, returning an object containing _only_ the props named in your spec, but this function here will apply the spec and then merge back it into the original input.
 *
 * @function
 * @name mergeSpec
 * @sig {k: v} -> {k: v} -> {k: v}
 * @param {Object} spec An object whose values are functions which expect the entire object to be fed in as input.  This is identical to the input you'd pass into applySpec()
 * @param {Object} input An object to be mapped over (transformed)
 * @returns {Object} The input object with all the transform functions applied at the named keys
 */

var mergeSpec = curry(function (spec, value) {
  return combine(value, mapSpec(spec, value));
});

exports.alwaysEvolve = alwaysEvolve;
exports.applyWholeObjectTransforms = applyWholeObjectTransforms;
exports.combine = combine;
exports.default = makeShaper;
exports.evolveSpec = evolveSpec;
exports.keepAndShape = keepAndShape;
exports.keeper = keeper;
exports.mapSpec = mapSpec;
exports.mergeSpec = mergeSpec;
exports.removeAndShape = removeAndShape;
exports.remover = remover;
exports.shape = makeShaper;
exports.shapeLoosely = shapeLoosely;
exports.shapeStrictly = shapeStrictly;
exports.shapeline = shapeline;

Object.defineProperty(exports, '__esModule', { value: true });

})));
